% function GCV = GCV_of_phi(F, lambda, sd, beta, myparam)
% 
% The function GCV is the function we need to minimize in order to compute
% the optimal parameter for the spectral filters based on the Generalized
% Cross Validation method.
%
% This function is equal to the norm-squared of the residual
%         divided by the trace of (I - Phi) squared.
%
% -------------------------------------------------------------------------
% Input:
% -------------------------------------------------------------------------
% F         : filter
% lambda    : The parameter of the filter
% sd        : Singular values of the bluring matrix, descending order
% beta      : Right hand side in the singular value space
% myparam   : A structure containing extra information needed
%             to construct the filter. (not used)
% -------------------------------------------------------------------------
% Output:
% -------------------------------------------------------------------------
% GCV         : The function GCV(phi)
% -------------------------------------------------------------------------
%
% For the derivation of GCV(phi) see Hansen et al. "Deblurring Images.
% Matrices, Spectra, and Filtering, SIAM Philadelphia, 2006
% 
% This is code to accompany:
% Near-Optimal Spectral Filtering and Error Estimation
% for Solving Ill-Posed Problems,
% Viktoria Taroudaki and Dianne P. O'Leary,
% University of Maryland, 2015.


function GCV = GCV_of_phi(F, lambda, sd, beta, myparam)

phi = feval(F, lambda, sd, beta, myparam);

GCV_n= norm( (1-phi) .* beta )^2;
GCV_d= sum( 1-phi )^2;

GCV=GCV_n/GCV_d;
