% function g = SOF_of_phi(F, lambda, sd, beta, myparam)
% 
% The function g is the function we need to minimize in order to compute
% the optimal parameter for the spectral filters based on the statistically
% optimal filter (SOF) method.
%
% Adding myparam.est_std^2 * sum(1./sd(1:myparam.kPicard-1)
% to g gives an estimate of || xtrue - x ||^2 in the subspace
% spanned by the first myparam.kPicard-1 singular values.
%
% -------------------------------------------------------------------------
% Input:
% -------------------------------------------------------------------------
% F         : A function handle so that F(lambda, sd, beta, myparam)
%             returns the n values of the filter, 
%             phi_lambda (sd(j)), j=1:n.
% lambda    : The parameter of the filter
% sd        : Singular values of the bluring matrix, descending order
% beta      : Right hand side in the singular value space
% myparam   : A structure containing extra information needed
%             to construct the filter, including myparam.kPicard,
%             the Picard parameter.
% -------------------------------------------------------------------------
% Output:
% -------------------------------------------------------------------------
% g         : The function g(phi)
% -------------------------------------------------------------------------
%
% This is code to accompany:
% Near-Optimal Spectral Filtering and Error Estimation
% for Solving Ill-Posed Problems,
% Viktoria Taroudaki and Dianne P. O'Leary,
% University of Maryland, 2015.

function g = SOF_of_phi(F, lambda, sd, beta, myparam)

phi_v = feval(F, lambda, sd, beta, myparam);

ind = 1:myparam.kPicard-1;

phihat = 1 - phi_v(ind);

myvec = ( phihat .* beta(ind) ).^2 ;

myvec = myvec - 2 * myparam.est_std^2 * phihat;

g = sum(myvec ./sd(ind).^2);

