% function [kPicard, est_mean_err, est_std_err] = estimate_Picard_Param(beta)
%
% This function estimates the Picard parameter for an ill-posed problem,
% assuming that the noise in the data is Gaussian.
%
% It also estimates the mean and standard deviation of the noise.
%
% The Picard parameter is the index k beyond which the data, expressed
% in the coordinate system of the left singular vectors, is noise.
%
% -------------------------------------------------------------------------
% Input:
% -------------------------------------------------------------------------
% beta      : Data vector in the coordinate system of the
%             singular vectors: i.e., beta(i) = u_i^T b, where 
%             b is a vector containing the measurements and
%             u_i is the left singular vector corresponding to the i-th
%             largest singular value.
%
% -------------------------------------------------------------------------
% Output:
% -------------------------------------------------------------------------
% kPicard     : The Picard parameter:
%               Elements beta(kPicard),...beta(n) were judged to be 
%               Gaussian noise.
%               If no such subsequence is found, kPicard is set to
%               length(beta)+1.
% est_mean_err: The estimated mean of the noise.
% est_std_err : The estimated standard deviation of the noise.
% -------------------------------------------------------------------------
%
% This is code to accompany:
% Near-Optimal Spectral Filtering and Error Estimation
% for Solving Ill-Posed Problems,
% Viktoria Taroudaki and Dianne P. O'Leary,
% University of Maryland, 2015.

function [kPicard, est_mean_err, est_std_err] = estimate_Picard_Param(beta)

n = length(beta);

start=n-3;
lohi=[start,n]; % lillietest needs a sequence of length at least 4.

% This loop stops when 10 consecutive sequences are judged not to be
% normally distributed.
% (There are probably better ways to do this!)

while ((lohi(2)-lohi(1)<=9)&&(start>0))

   % Use the Lilliefors Test on the subsequence of beta starting with 
   % beta(start) to see if the subsequence is normally distributed.

   res=lillietest(beta(start:n));  

   if res==0 % The subsequence passed the Lilliefors test.
       lohi(2)=start-1;
       lohi(1)=start-1;
   elseif res==1 % The subsequence does not appear to be normally distributed.
       lohi(1)=start-1;
   end

   start=start-1;
end

kPicard = lohi(2)+1;

est_std_err  =  std(beta(kPicard:end));
est_mean_err = mean(beta(kPicard:end));


