% function s = eval_spline(pt, spline)
%
% This function evaluates a cubic spline, setting 
%          s(i) = the spline evaluated at pt(i).
%               = m(j)*(x(j+1)-pt(i))^3/(6*h(j+1))
%                 + m(j+1)*(pt(i)-x(j))^3/(6*h(j+1))
%                 + a(j)*(pt(i)-x(j))
%                 + b(j) , when x(j) <  pt(i) <= x(j+1);
%          s(i) = 0 otherwise.
% The values m, a, b, x, and h are parameters, stored in the structure
% "spline", that define the spline.
% Values are then truncated to lie within the interval [0,1].
% -------------------------------------------------------------------------
% Input:
% -------------------------------------------------------------------------
% pt        : A vector of evaluation points.
% spline.m  : The 2nd derivatives at the g knots.
% spline.a  : g-1 parameters defining the spline.
% spline.b  : g-1 parameters defining the spline.
% spline.x  : The g knots for the spline.                               
% spline.h  : The g-1 distances between adjacent knots.
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
% Output:
% -------------------------------------------------------------------------
% s         : The values of the spline at the points in pt.
% -------------------------------------------------------------------------
%
% This version is most efficient if the points in pt are arranged
% in ascending or descending order.
%
% This is code to accompany:
% Near-Optimal Spectral Filtering and Error Estimation
% for Solving Ill-Posed Problems,
% Viktoria Taroudaki and Dianne P. O'Leary,
% University of Maryland, 2015.

function s = eval_spline(pt, spline)

[np,mp]= size(pt);

g = length(spline.m);

np = max(np, mp);  % allows pt to be a row or column, but not a matrix.

s = zeros(size(pt));

j = g-1;
for i=1:np
    j = min(j,g-1);
    if ((pt(i)<=spline.x(j+1)) && (pt(i)>spline.x(j)))
             s(i)=spline.m(j)*(spline.x(j+1)-pt(i))^3/(6*spline.h(j+1))...
                +spline.m(j+1)*(pt(i)-spline.x(j))^3/(6*spline.h(j+1))...
                +spline.a(j)*(pt(i)-spline.x(j))...
                +spline.b(j);
    else
      for j=g-1:-1:1
        if ((pt(i)<=spline.x(j+1)) && (pt(i)>spline.x(j)))
             s(i)=spline.m(j)*(spline.x(j+1)-pt(i))^3/(6*spline.h(j+1))...
                +spline.m(j+1)*(pt(i)-spline.x(j))^3/(6*spline.h(j+1))...
                +spline.a(j)*(pt(i)-spline.x(j))...
                +spline.b(j);
            break;
        end
      end
   end
end

% Truncate to [0,1].

s = min(1,s);
s = max(0,s);
