% function [xhat, filter, error_est, lambda_min, g_min] = ...
%       find_opt_filter(F, sd, beta, opt_type, myparam)
%
% This function finds the optimal filter for an ill-posed problem
%               A x = b,
% expressed in the coordinate system of the singular value decomposition
% as
%               S xhat = beta,   where S = diag(sd).
%
% "Optimality" is determined by finding lambda_min so that 
% filter(lambda_min) minimizes a function g, defined below.
%
% Instead of the naive solution inv(S)*beta, we compute 
%               xhat = Phi * inv(S) * beta, 
% where Phi = diag(filter(lambda_min)).
%
% -------------------------------------------------------------------------
% Input:
% -------------------------------------------------------------------------
% F         : A function handle so that F(lambda, sd, beta, myparam)
%             returns the n values of the filter evaluated at the
%             singular values sd. 
%             phi_lambda (sd(j)), j=1:n.
% sd        : Singular values of the blurring matrix in descending order
%             (vector of length n).
% beta      : Data vector in the coordinate system of the
%             singular vectors: i.e., beta(i) = u_i^T b, where 
%             b is a vector containing the measurements and
%             u_i is the left singular vector corresponding to the i-th
%             largest singular value.
% opt_type  : Specifies what optimization criterion is used.
%             opt_type = 1: Determine the statistically optimal SOF) filter.
%             opt_type = 2: Determine the filter using GCV.
%             opt_type = 3: Determine the filter using the Discrepancy Principle.
% myparam   : A structure containing extra information needed
%             to construct the filter.
%             myparam.est_std: the estimated standard deviation of the noise
%                  in beta. 
%                  (For Gaussian noise, it can be computed by 
%                  estimate_Picard_Param.m) 
%             myparam.nCont: The number of continuous parameters in lambda.
%             myparam.Cont_Up: Upper bounds on these parameters.
%             myparam.Cont_Lo: Lower bounds on these parameters.
%             myparam.nDis: The number of discrete parameters in lambda.
%             myparam.Dis_vals: myparam.Dis_vals{j} is a vector containing
%                  the possible values for the j-th discrete parameter.
%             myparam.est_std: The estimated standard deviation of the
%                  (For Gaussian noise, it can be computed by 
%                  estimate_Picard_Param.m) 
%             myparam.kPicard: the Picard parameter, the index beyond which
%                  the elements of beta can be assumed to be noise.
%                  (For Gaussian noise, it can be computed by 
%                  estimate_Picard_Param.m) 
%
% Note: myparam.Cont_Up and myparam.Cont_Lo are not used if myparam.nCont > 1
% since that would require access to Matlab's Optimization Toolbox, but
% consider modifying the code to use fmincon in place of fminsearch in this case.
% -------------------------------------------------------------------------
% Output:
% -------------------------------------------------------------------------
% xhat       : The computed solution (vector of length n).
% filter     : The optimal filter (vector of length n).
% error_est  : The estimated error, the estimated 2-norm of the true 
%              solution minus the computed solution.
% lambda_min : The optimal parameters for the filter.
% g_min      : The minimum function value.
% -------------------------------------------------------------------------
%
% This is code to accompany:
% Near-Optimal Spectral Filtering and Error Estimation
% for Solving Ill-Posed Problems,
% Viktoria Taroudaki and Dianne P. O'Leary,
% University of Maryland, 2015.

function [xhat, filter, error_est, lambda_min, g_min] = ...
      find_opt_filter(F, sd, beta, opt_type, myparam)

dvalbest = [];
cvalbest = [];
dval = [];
cval = [];

if myparam.nCont > 1  % then compute an initial guess.
  for j=1:myparam.nCont
      if      isinf(myparam.Cont_Lo(j)) &  isinf(myparam.Cont_Up(j))
              init_point(j) = rand(1);
      elseif  isinf(myparam.Cont_Lo(j)) & ~isinf(myparam.Cont_Up(j))
              init_point(j) = min(1,myparam.Cont_Up/2);
      elseif ~isinf(myparam.Cont_lo(j)) &  isinf(myparam.Cont_Up(j))
              init_point = max(-1, myparam_Cont_Lo/2);
      else
         init_point = .5*(myparam.Cont_Lo(j) + myparam.Cont_Up(j));
      end
   end
end
      
% Case 1: No discrete parameters.
% Minimize using fminbnd if there is 1 continuous parameter,
%                fminsearch if there are more than 1.

if myparam.nDis==0
   if myparam.nCont==1
      if opt_type==1
         [cvalbest, gvalbest] = fminbnd( ...
            @(lambda) SOF_of_phi(F, lambda, sd, beta, myparam), ...
                                 myparam.Cont_Lo, myparam.Cont_Up);
      elseif opt_type==2
         [cvalbest, gvalbest] = fminbnd(...
            @(lambda) GCV_of_phi(F, lambda, sd, beta, myparam), ...
                                 myparam.Cont_Lo, myparam.Cont_Up);
      elseif opt_type==3
         [cvalbest, gvalbest] = fminbnd(...
            @(lambda) DP_of_phi(F, lambda, sd, beta, myparam), ...
                                 myparam.Cont_Lo, myparam.Cont_Up);
      else
         [cvalbest, gvalbest] = fminbnd(...
            @(lambda) OPT_of_phi(F, lambda, sd, beta, myparam), ...
                                 myparam.Cont_Lo, myparam.Cont_Up);
      end
   else  
      if opt_type==1
         [cvalbest, gvalbest] = fminsearch(...
            @(lambda) SOF_of_phi(F, lambda, sd, beta, myparam), ...
                                init_point );                    
      elseif opt_type==2
         [cvalbest, gvalbest] = fminsearch(...
            @(lambda) GCV_of_phi(F, lambda, sd, beta, myparam),  ...
                                 init_point );                    
      elseif opt_type==3
         [cvalbest, gvalbest] = fminsearch(...
            @(lambda) DP_of_phi(F, lambda, sd, beta, myparam), ...
                                 init_point );                    
      else
         [cvalbest, gvalbest] = fminsearch(...
            @(lambda) OPT_of_phi(F, lambda, sd, beta, myparam), ...
                                 init_point );                    
      end
   end
else

% Case 2: At least one discrete parameter.
% Find the minimum value for each set of discrete parameter values
% and save the best.

   for j=1:myparam.nDis
      nv(j) = length(myparam.Dis_vals{j});
      dval(j) = myparam.Dis_vals{j}(1);
   end
   ndval = prod(nv);
   dvalbest = dval;
   gvalbest = Inf;
   index = ones(1,myparam.nDis);
   index(1) = 0;
   for jj = 1:ndval

       % Generate next set of discrete values, dval.

       j = 1;
       index(j) = index(j) + 1;
       
       while index(j) > nv(j)
          index(j) = 1;
          dval(j) = myparam.Dis_vals{j}(1);
          j = j+1;
          index(j) = index(j) + 1;
       end

       dval(j)  = myparam.Dis_vals{j}(index(j));

       % Find the minimizer for this set of values.

       if myparam.nCont==0       % ... just evaluate the function.
          if opt_type==1
             g_min =   SOF_of_phi(F, dval, sd, beta, myparam);
          elseif opt_type==2
             g_min = GCV_of_phi(F, dval, sd, beta, myparam);
          elseif opt_type==3
             g_min =  DP_of_phi(F, dval, sd, beta, myparam);
          else
             g_min =  OPT_of_phi(F, dval, sd, beta, myparam);
          end   
       elseif myparam.nCont==1   % ... use fminbnd to minimize.
          if opt_type==1
             [cval, g_min] = fminbnd(...
                @(lambdac) SOF_of_phi(F, [dval,lambdac], sd, beta, myparam), ...
                                 myparam.Cont_Lo, myparam.Cont_Up);
          elseif opt_type==2
             [cval, g_min] = fminbnd(...
                @(lambdac) GCV_of_phi(F, [dval,lambdac], sd, beta, myparam), ...
                                 myparam.Cont_Lo, myparam.Cont_Up);
          elseif opt_type==3
             [cval, g_min] = fminbnd(...
                @(lambdac) DP_of_phi(F, [dval,lambdac], sd, beta, myparam), ...
                                 myparam.Cont_Lo, myparam.Cont_Up);
          else
             [cval, g_min] = fminbnd(...
                @(lambdac) OPT_of_phi(F, [dval,lambdac], sd, beta, myparam), ...
                                 myparam.Cont_Lo, myparam.Cont_Up);
          end
       else                     % ... use fminsearch to minimize.
               %SO FAR BOUNDS NOT INCLUDED HERE!!!!!!!!!!!!!!!`
          if opt_type==1
             [cval, g_min] = fminsearch(...
                @(lambdac) SOF_of_phi(F, [dval,lambdac], sd, beta, myparam), ...
                                init_point);
          elseif opt_type==2
             [cval, g_min] = fminsearch(...
                @(lambdac) GCV_of_phi(F, [dval,lambdac], sd, beta, myparam),  ...
                                init_point );
          elseif opt_type==3
             [cval, g_min] = fminsearch(...
                @(lambdac) DP_of_phi(F, [dval,lambdac], sd, beta, myparam), ...
                                init_point );
          else
             [cval, g_min] = fminsearch(...
                @(lambdac) OPT_of_phi(F, [dval,lambdac], sd, beta, myparam), ...
                                init_point );
          end
          init_point = cval;
       end

       % Save this set of values if it is the best so far.

       if g_min < gvalbest
          dvalbest = dval;
          cvalbest = cval;
          gvalbest = g_min;
       end
   end % for jj
       
end

% Return the estimated optimal filter, the filtered solution xhat,
% and the error estimate error_est.

lambda_min = [dvalbest, cvalbest];
g_min = gvalbest;
filter  = F(lambda_min, sd(:), beta(:), myparam);

xhat =(beta./sd).*filter;
   
ind = 1:myparam.kPicard-1;
phihat = 1 - filter(ind);

ind2 = find(filter(ind) < .5);

myvec = zeros(myparam.kPicard-1,1);

myvec(ind2) = ( filter(ind2).*beta(ind2) ).^2;

ind3 = setdiff(ind,ind2);

myvec(ind3) = (2*filter(ind3) - 1) .* myparam.est_std^2 ...
               + ( phihat(ind3).*beta(ind3) ).^2;

error_est = sqrt ( sum ( myvec ./ (sd(ind).^2) ));
