% function phi=phi_Hybrid(lambda, sd,  beta, myparam)
% 
% This routine computes a hybrid filter (combination of the TSVD and the 
% Tikhonov filters) for numerical solution of ill-posed problems, including 
% reconstruction of images that have been blurred.
%
% The filter is defined as: 
% 
%    phi(j) = 1                            if 0 < j <=  kDis,
%           = sd^2(j)/(sd^2(j)+Tparam)     if kDis < j < kPicard,
%           = 0                            if kPicard <= j,
% where sd are the singular values in non-increasing order and 
% kPicard comes through myparam.
% -------------------------------------------------------------------------
% Input:
% -------------------------------------------------------------------------
% lambda    : The parameter defining the filter = [kDis, Tparam].
% sd        : Singular values of the blurring matrix in non-increasing order.
% beta      : Right hand side in the coordinate system of the
%             singular vectors. (not used)
% myparam   : A structure containing extra information needed
%             to construct the filter: myparam.kPicard.
% -------------------------------------------------------------------------
% Output:
% -------------------------------------------------------------------------
% phi       : The Hybrid filter (vector of length nphi).
% -------------------------------------------------------------------------
%
% This is code to accompany:
% Near-Optimal Spectral Filtering and Error Estimation
% for Solving Ill-Posed Problems,
% Viktoria Taroudaki and Dianne P. O'Leary,
% University of Maryland, 2015.


function phi=phi_Hybrid(lambda, sd,  beta, myparam)

kDis    = lambda(1);
Tparam  = lambda(2);
kPicard = myparam.kPicard;

n = length(sd);
phi = zeros(n,1);
phi(1:kDis) = 1;
irange = kDis+1:kPicard-1;
phi(irange)=sd(irange).^2./(sd(irange).^2+Tparam);

