% function [phi]=phi_TSCM(lambda, sd, beta, myparam)
% 
% This routine computes the TSCM filter for numerical 
% solution of ill-posed problems, including 
% reconstruction of images that have been blurred.
%
% The filter is defined as: 
% 
%    phi(j) = 1 if |beta(j)| > lambda and j < kPicard,
%           = 0 otherwise,
%
% where kPicard is sent through the structure myparam.
%
% The original TSCM, without use of the Picard parameter, is
% defined in
% B.W. Rust, Truncating the singular value decomposition for 
% ill-posed problems, NISTIR 6131, U.S. Dept. of Commerce, (1998).
%           
% -------------------------------------------------------------------------
% Input:
% -------------------------------------------------------------------------
% lambda    : The parameter defining the filter.
% sd        : Singular values of the blurring matrix in non-increasing
%             order.
% beta      : Right hand side in the coordinate system of the
%             singular vectors.
% myparam   : A structure containing extra information needed
%             to construct the filter: the Picard parameter
%             myparam.kPicard 
% -------------------------------------------------------------------------
% Output:
% -------------------------------------------------------------------------
% phi       : The TSCM filter (vector with length(sd)).
% -------------------------------------------------------------------------
%
% This is code to accompany:
% Near-Optimal Spectral Filtering and Error Estimation
% for Solving Ill-Posed Problems,
% Viktoria Taroudaki and Dianne P. O'Leary,
% University of Maryland, 2015.


function [phi]=phi_TSCM(lambda, sd, beta, myparam)

phi=zeros(size(sd));
ind = find(abs(beta(1:myparam.kPicard-1)) >= lambda);
phi(ind) = 1;

