% function [phi] = phi_TTik(lambda, sd,  beta, myparam)
% 
% This routine computes the Truncated Tikhonov (TTik) filter for numerical 
% solution of ill-posed problems, including 
% reconstruction of images that have been blurred.
%
% The filter is defined as: 
% 
%    phi(j) = sd^2(j)/(sd^2(j)+lambda) if j < kPicard
%           = 0                        otherwise
%           
% where sd are the singular values in non-increasing order and kPicard
% is the Picard parameter.
%
% -------------------------------------------------------------------------
% Input:
% -------------------------------------------------------------------------
% lambda    : The parameter defining the filter.
% sd        : Singular values of the blurring matrix in non-increasing order.
%             (vector of length nphi).
% beta      : Right hand side in the coordinate system of the
%             singular vectors. (not used)
% myparam   : A structure containing extra information needed
%             to construct the filter: myparam.kPicard.
% -------------------------------------------------------------------------
% Output:
% -------------------------------------------------------------------------
% phi       : The Truncated Tikhonov filter, a vector with length(sd).
% -------------------------------------------------------------------------
%
% This is code to accompany:
% Near-Optimal Spectral Filtering and Error Estimation
% for Solving Ill-Posed Problems,
% Viktoria Taroudaki and Dianne P. O'Leary,
% University of Maryland, 2015.

function [phi] = phi_TTik(lambda, sd,  beta, myparam)

k = myparam.kPicard;
phi = zeros(size(sd));
phi(1:k-1) = (sd(1:k-1).^2)./(sd(1:k-1).^2 + lambda);
