% function [phi,spline]=phi_spline(lambda, sd, beta, myparam)
%
% This routine computes a spline filter 
% for numerical solution of ill-posed problems, including 
% reconstruction of images that have been blurred.
%
% Given a set of knots x(j),  j=1:nknots+1, 
% define h_{j+1}=x_{j+1}-x_j, j=1:nknots.  
% Then the filter is defined by a cubic spline with knots x, 
% and it is evaluated as
%
%             phi(i)=spline.m(j)*(x(j+1)-sd(i))^3/(6*h(j+1))+...
%                spline.m(j+1)*(sd(i)-x(j))^3/(6*h(j+1))+...
%                spline.a(j)*(sd(i)-x(j))+...
%                spline.b(j) , when x(j) <  sd(i) <= x(j+1)
%
% for some parameters 
%        m(1), ... , m(nknots-1), m(nknots),
%        a(1), ... , a(nknots-1),
%        b(1), ... , b(nknots-1).
% -------------------------------------------------------------------------
% Input:
% -------------------------------------------------------------------------
% lambda    : The parameter defining the filter.
% sd        : Singular values of the blurring matrix in non-increasing order.
% beta      : Right hand side in the coordinate system of the
%             singular vectors. (not used)
% myparam   : A structure containing extra information needed
%             to construct the filter: 
% myparam.kPicard : Picard parameter. Singular values smaller than this
%                   are ignored.
% myparam.nCont   : Number of knots in the spline. (Also the number of 
%                   continuous parameters in the problem).
% myparam.scaling : = 1 for knots that are equally spaced on a linear scale
%                   = any other number for knots that are equally spaced on
%                     a logarithmic scale.
% myparam.cas     : = Specifies the boundary conditions for the spline.
%                        Case 1: m_1=0,      m_nknots=0.
%                        Case 2: m_1=0,      phi(x_nknots)=1.
%                        Case 3: phi(x_1)=0, m_nknots=0.
%                        Case 4: phi(x_1)=0, phi(x_nknots)=1.
%                                Case 4 is used in our work.
% 
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
% Output:
% -------------------------------------------------------------------------
% phi       : The Cubic Spline Filter (vector of length nphi).
% spline    : A structure containing m, a, b, x, and h, defining the spline.
% -------------------------------------------------------------------------
%
% This is code to accompany:
% Near-Optimal Spectral Filtering and Error Estimation
% for Solving Ill-Posed Problems,
% Viktoria Taroudaki and Dianne P. O'Leary,
% University of Maryland, 2015.

function [phi, spline]=phi_spline(lambda, sd, beta, myparam)

% The number of continuous parameters is the number of knots.

nknots=myparam.nCont;
n=length(sd);

% We choose the knots depending on sd(1) and sd(kPicard).

k = min(n,myparam.kPicard);

% The knots x can either be equally spaced on a log scale 
% or equally spaced on a linear scale (myparam.scaling = 1).

if myparam.scaling==1
    var = sd;
else
    var = log(sd);
end
spline.x=linspace(var(k), var(1), myparam.nCont);

spline.h=zeros(nknots,1);
for i=1:nknots-1
    spline.h(i+1)=spline.x(i+1)-spline.x(i);
end

%%

% Extract the g parameters from the lambda vector and store them as
% spline.a(1), spline.b(1), and spline.m(2:nknots-1).

spline.m(2:nknots-1) = lambda(1:nknots-2);
spline.a(1)     = lambda(nknots-1);
spline.b(1)     = lambda(nknots);

% Set the values of the other a, b, and m parameters, depending on the 
% boundary conditions.

if ((myparam.cas==3)||(myparam.cas==4))
    spline.m(1)=-spline.b(1)*6/(spline.h(2))^2;
elseif ((myparam.cas==1)||(myparam.cas==2))
    spline.m(1)=0;
end

for i=2:nknots-1
    spline.a(i)=spline.a(i-1)+(spline.h(i+1)+spline.h(i))*spline.m(i)/2;
    spline.b(i)=spline.b(i-1)+spline.h(i)*spline.a(i-1) ...
         +(spline.h(i)^2-spline.h(i+1)^2)*spline.m(i)/6;
end

if ((myparam.cas==1)||(myparam.cas==3))
    spline.m(nknots)=0;
elseif ((myparam.cas==2)||(myparam.cas==4))
    spline.m(nknots)=6*(1-spline.b(nknots-1) ...
         -spline.a(nknots-1)*spline.h(nknots))/(spline.h(nknots))^2;
end

% Compute the filter by evaluating the spline at var(1), ... var(kPicard).

phi=zeros(n,1);
phi(1:k) = eval_spline(var(1:k), spline);

