
%% This program deblurs an image using a filter based on the 
% singular values of the blurring matrix, with parameters
% chosen in an optimal way.
%
% The model is Ar X Ac' + E = B,
%
% where B is the (known) blurred image,
%       X is the (unknown) true image,
%       E is the (unknown) Gaussian error,
%       Ar and Ac are the (known) blurring matrices, derived from the
%          point spread function for the blur.
%
% An alternate model is A x + e = b,
%
% where b is the (known) blurred image arranged as a vector,
%       x is the (unknown) true image vector,
%       e is the (unknown) Gaussian error vector,
%       A is the (known) blurring matrix.
%
% The program can also be used for ill-posed problems that are not
% related to images, as long as A and b are known and the noise
% is assumed to be Gaussian.
%
% Any of these fitering methods can be chosen:
%
%   method==1  Truncated singular value decomposition (TSVD) without Picard parameter 
%   method==2  Tikhonov without Picard parameter
%   method==3  Truncated Tikhonov
%   method==4  TSVD with Picard parameter
%   method==5  Truncated singular component method (TSCM)
%   method==6  Hybrid Truncated Tikhonov-TSVD method
%   method==7  Spline Filter with knots linearly spaced
%   method==8  Spline Filter with knots log spaced
%
% Any of these methods can be used for choosing the parameters for the filter:
%     SG = 1 for the Statistically Optimal Filter (SOF),
%     SG = 2 for generalized cross-validation (GCV),
%     SG = 3 for the discrepancy principle (DP).
%
% This is code to accompany:
% Near-Optimal Spectral Filtering and Error Estimation
% for Solving Ill-Posed Problems,
% Viktoria Taroudaki and Dianne P. O'Leary,
% University of Maryland, 2015.
%
% Initialization:

% Choose the method to be run: meth = [method,SG].

meth= [7,1];

% Input the problem data: a blurring matrix A and the blurred image.
% In this example, the blurred image is stored as variable B in the 
% file blurred.mat and the blur is Gaussian.

load('blurred')
dims = size(B);

[P,center]=psfGauss([3,3]);
P=padPSF(P, dims(1),dims(2));

[Ar, Ac]=kronDecomp(P, center);

% Find the singular values of the blurring matrix.
% For the alternate model, just set 
%     [U,S,V] = svd(A);
% In our example, A is the Kronecker product of Ar and Ac,
% so we take the SVDs of both matrices.

[Ur, Sr, Vr] = svd(Ar);
[Uc, Sc, Vc] = svd(Ac);

n=dims(1)*dims(2);

% Beta is B in the coordinate system defined by the blur.
% For the alternate model, just set
%   beta = U' * b(:);

Beta=Uc'*B*Ur;
beta=reshape(Beta,n,1);

% Compute the singular values of the blurring operator,
% stored in an nxn matrix Sd and in vector sd.
% For the alternate model, just set
%   sd = diag(S);

Sd=diag(Sc)*(diag(Sr))'; 
sd=reshape(Sd, n, 1);

% Prepare to estimate the Picard parameter, the index k beyond
% which the singular values can be ignored, because the components
% of beta are dominated by noise.

disp(' ')
fprintf('Beginning Picard parameter estimation. \n')

% Sort the singular values sd in descending order, obtaining
% sd_sorted and indices ind_sort.
% Reorder beta to obtain beta_sorted.
% For the alternate model, just set
%    sd_sorted = sd;
%    ind_sort = [1:n];
%    beta_sorted = beta;

[sd_sorted, ind_sort]=sort(sd,'descend');

beta_sorted=beta(ind_sort);

%% Estimate the Picard Parameter using the Lilliefors test.

[k, est_mean_err, est_std_err] = estimate_Picard_Param(beta_sorted);

myparam.delta_k = sd_sorted(k);
myparam.kPicard = k;

cropped = n-k+1;

fprintf('\n Picard parameter is sigma(%d) = %f error is %f. \n', ...
         k, myparam.delta_k)
fprintf('\n Estimated standard deviation of the error is %f.\n', ...
         est_std_err)

% Set target for Discrepancy Principle residual.
% You might want to change the "2" to another small integer.

tau_DP = 2;
myparam.est_std = est_std_err;
myparam.DPtarget = tau_DP * est_std_err * sqrt(n); 

%%%% Run the chosen method. 

% In the myparam structure, set:
%   nDis = number of discrete parameters,
%   Dis_vals{j} = array of values the parameters can take on, j=1:Dis,
%   nCont = number of continuous parameters,
%   Cont_Lo = array of lower bounds for the continuous parameters,
%   Cont_Up = array of upper bounds for the continuous parameters.

method = meth(1);
SG     = meth(2);

if method==1 % TSVD, without Picard parameter 
    myparam.kPicard = n+1;
    F=@phi_TSVD;
    myalg = 'TSVD';
    myparam.nDis=1;
    myparam.Dis_vals{1} = [1:n];
    myparam.nCont=0;
elseif method==2 % Tikhonov, without Picard parameter
    myparam.kPicard = n+1;
    F=@phi_TTik;
    myalg = 'TIK';
    myparam.nDis=0;
    myparam.nCont=1;
    myparam.Cont_Lo = 0;
    myparam.Cont_Up = sd_sorted(1)^2;
elseif method==3 % Truncated Tikhonov
    F=@phi_TTik;
    myalg = 'TIKk';
    myparam.nDis=0;
    myparam.nCont=1;
    myparam.Cont_Lo = 0;
    myparam.Cont_Up = sd_sorted(1)^2;
elseif method==4 % TSVD, with Picard parameter
    F=@phi_TSVD;
    myalg = 'TSVDk';
    myparam.nDis=1;
    myparam.Dis_vals{1} = [1:k-1];
    myparam.nCont=0;
elseif method==5 % TSCM
    F=@phi_TSCM;
    myalg = 'TSCM';
    myparam.nDis=1;
    myparam.Dis_vals{1} = abs(beta_sorted(1:k-1));
    myparam.nCont=0;
elseif method==6  % Hybrid Truncated Tikhonov- TSVD method
    F=@phi_Hybrid;
    myalg = 'HYBR';
    myparam.nDis=1;
    myparam.Dis_vals{1} = [1:k-1];
    myparam.nCont=1;
    myparam.Cont_Lo = 0;
    myparam.Cont_Up = sd_sorted(1)^2;
elseif method==7 % Spline Filter with nknots knots equally spaced
                 % (nknots unknowns, 2nknots-2 dependent parameters)
    nknots=5;
    myalg = 'SPL_lin';
    F=@phi_spline;
    myparam.nDis=0;
    myparam.nCont=nknots;
    myparam.Cont_Lo = -Inf*ones(1,nknots);
    myparam.Cont_Up =  Inf*ones(1,nknots);
    myparam.scaling=1;
    % The choices for boundary conditions for the spline are: 
    % Case 1: m_1=0,      m_nknots=0.
    % Case 2: m_1=0,      phi(x_nknots)=1.
    % Case 3: phi(x_1)=0, m_nknots=0.
    % Case 4: phi(x_1)=0, phi(x_n)=1.
    % where phi is the spline we are computing.
    myparam.cas=4;
elseif method==8 % Spline Filter with nknots knots equally spaced on log scale
                 % (nknots unknowns, 2nknots-2 dependent parameters)
    nknots=5;
    myalg = 'SPL_log';
    F=@phi_spline;
    myparam.nDis=0;
    myparam.nCont=nknots;
    myparam.Cont_Lo = -Inf*ones(1,nknots);
    myparam.Cont_Up =  Inf*ones(1,nknots);
    myparam.scaling=0;
    myparam.cas=4;
end

% Calculate the solution Xnew for this method.

disp(' ')
disp('Beginning the estimation of the solution.')

[xhat_sorted, filter_sorted, error_est, lambda_min, g_min] = ...
     find_opt_filter(F, sd_sorted, beta_sorted, SG, myparam);

lambda_opt{method,SG} = lambda_min;
g_opt(method,SG) = g_min;

xhat(ind_sort) = xhat_sorted;

% For the alternate model, just set
% xnew = V * xhat;         (changed 07/22/16)

Xnew = Vc * reshape(xhat,dims(1),dims(2)) * Vr';
  
% Display the results.

method_label{1} = 'TSVD, without Picard parameter';
method_label{2} = 'Tikhonov, without Picard parameter';
method_label{3} = 'Truncated Tikhonov'; 
method_label{4} = 'TSVD, with Picard parameter';
method_label{5} = 'TSCM';
method_label{6} = 'Hybrid Truncated Tikhonov - TSVD';
method_label{7} = 'Spline Filter, knots equally spaced';
method_label{8} = 'Spline Filter, knots equally spaced on log scale';

SG_label{1} = 'SOF';
SG_label{2} = 'GCV';
SG_label{3} = 'Discrepancy Principle';

disp(' ')
disp(sprintf('Using method %s,', ... 
     method_label{method} ))
disp(sprintf('      with regularization parameter chosen by %s.', ...
     SG_label{SG} ))

disp(' ')
disp(sprintf('The error is estimated to be %f', error_est))

disp(' ')
disp(sprintf('The results are shown in the figure.'))
disp(' ')

figure

subplot(1,2,1)
imagesc(B)
axis image
colormap(gray)
title('Blurred Image')

subplot(1,2,2)
imagesc(Xnew)
axis image
colormap(gray)
title('Reconstructed Image')

